#!/usr/bin/python
#
# Copyright 2018-2020 Polyaxon, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from tests.utils import BaseTestCase

from polyaxon.k8s.custom_resources.crd import get_custom_object


class TestCRD(BaseTestCase):
    def test_get_custom_object(self):
        crd = get_custom_object(
            namespace="default",
            resource_name="foo",
            kind="job",
            api_version="v1",
            labels={"foo": "bar"},
            custom_object={"some_spec": {"foo": "bar"}},
        )
        assert crd["kind"] == "job"
        assert crd["apiVersion"] == "v1"
        assert crd["metadata"].name == "foo"
        assert crd["metadata"].labels == {"foo": "bar"}
        assert crd["metadata"].namespace == "default"
        assert crd["some_spec"] == {"foo": "bar"}
